#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         CONFORMANCE
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT
set reportfile    report.csv

#
# test case Name, category, description, GUID...
#
CaseGuid        F3DEAC0E-75C1-45e9-93FA-9F708BD30015
CaseName        Receive.Conf4.Case1
CaseCategory    MNP
CaseDescription {Test the EFI_ABORTED conformance of MNP.Receive - Call        \
	               MNP.Cancel() to abort the Receive(). }
################################################################################

Include MNP/include/Mnp.inc.tcl

#
# Begin log ...
#
BeginLog
BeginScope  _MNP_RECEIVE_CONFORMANCE4_CASE1_

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local OS	Side Parameter"
#
UINTN                                 R_Status
UINTN                                 R_Handle
EFI_MANAGED_NETWORK_CONFIG_DATA       R_MnpConfData
EFI_MANAGED_NETWORK_COMPLETION_TOKEN  R_Token
UINTN                                 R_Context
#
# Create Child
#
MnpServiceBinding->CreateChild "&@R_Handle, &@R_Status"
GetAck
SetVar     [subst $ENTS_CUR_CHILD]  @R_Handle
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Receive - Conf - Create Child"                            \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Configure Child
#
SetMnpConfigData R_MnpConfData 0 0 0 FALSE FALSE FALSE FALSE FALSE FALSE TRUE
Mnp->Configure {&@R_MnpConfData, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Receive - Conf - Start Child"                              \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Call Receive with Token
#
SetVar  R_Context   1
GetVar  R_Context
while { $R_Context == 1 } {
  SetVar          R_Context 0
  BS->CreateEvent "$EVT_NOTIFY_SIGNAL, $EFI_TPL_CALLBACK, 1, &@R_Context,      \
                  &@R_Token.Event, &@R_Status"
  GetAck
  Mnp->Receive    "&@R_Token, &@R_Status"
  GetAck
  set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
  if { [string compare -nocase $assert "fail" ] == 0} {
    BS->CloseEvent "@R_Token.Event, &@R_Status"
    GetAck

    EndScope  _MNP_CANCEL_FUNCTION1_CASE1_
    EndLog
    exit
  }
  GetVar          R_Context
}
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Receive - Conf - Chk Ret Code"                             \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Call Cancel with Token1
#
Mnp->Cancel "&@R_Token, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Receive - Conf - Call Cancel() to aborts an                \
                asynchronous transmit or receive request."                     \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Check R_Context
#
GetVar  R_Context
set assert pass
if {1 != $R_Context} {
  set assert fail
}
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Receive - Conf - Check R_Context"                         \
                "Return R_Context - $R_Context, Expected R_Context - 1"

#
# Check R_Token.Status
#
GetVar  R_Token
set assert fail
if { ${R_Token.Status} == $EFI_ABORTED } {
  set assert pass
}
RecordAssertion $assert $MnpReceiveConf4AssertionGuid001                       \
                "Mnp.Receive - Conf - Chk Ret Code"                            \
                "ReturnStatus - ${R_Token.Status}, ExpectedStatus - $EFI_ABORTED" 

BS->CloseEvent "@R_Token.Event, &@R_Status"
GetAck

#
# Destroy Child
#
MnpServiceBinding->DestroyChild {@R_Handle, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "MnpSB.Receive - Conf - Destroy Child1"                        \
                "ReturnStatus - $R_Status,ExpectedStatus - $EFI_SUCCESS"

EndScope    _MNP_RECEIVE_CONFORMANCE4_CASE1_

#
# EndLog
#
EndLog
