#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         FUNCTION
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT

#
# test case Name, category, description, GUID...
#
CaseGuid          1677418B-245B-4c39-A968-58AED73FC178
CaseName          Routes.Conf1.Case1
CaseCategory      TCP
CaseDescription   {This case is to test the Conformance - EFI_NOT_STARTED.     \
                  -- The EFI TCPv4 Protocol instance has not been configured.}
################################################################################

Include TCP4/include/Tcp4.inc.tcl

proc CleanUpEutEnvironment {} {
  DestroyTcb
  DelEntryInArpCache

  Tcp4ServiceBinding->DestroyChild "@R_Handle, &@R_Status"
  GetAck

  EndScope _TCP4_SPEC_CONFORMANCE_
  EndLog
}

#
# Begin log ...
#
BeginLog

#
# BeginScope
#
BeginScope _TCP4_SPEC_CONFORMANCE_

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local OS Side Parameter"
#
UINTN                            R_Status
UINTN                            R_Handle
UINTN                            R_Context

BOOLEAN                          R_Routes_DeleteRoute
EFI_IPv4_ADDRESS                 R_Routes_SubnetAddress
EFI_IPv4_ADDRESS                 R_Routes_SubnetMask
EFI_IPv4_ADDRESS                 R_Routes_GatewayAddress

#
# Initialization of TCB related on OS side.
#
CreateTcb TCB $DEF_ENTS_IP_ADDR $DEF_ENTS_PRT $DEF_EUT_IP_ADDR $DEF_EUT_PRT

LocalEther  $DEF_ENTS_MAC_ADDR
RemoteEther $DEF_EUT_MAC_ADDR
LocalIp     $DEF_ENTS_IP_ADDR
RemoteIp    $DEF_EUT_IP_ADDR

#
# Add an entry in ARP cache.
#
AddEntryInArpCache

#
# Create Tcp4 Child.
#
Tcp4ServiceBinding->CreateChild "&@R_Handle, &@R_Status"
GetAck
SetVar     [subst $ENTS_CUR_CHILD]  @R_Handle
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Tcp4SBP.CreateChild - Create Child 1"                         \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Check Point: Call Tcp4.Routes(),to add an route entry in route table. 
#
SetVar          R_Routes_DeleteRoute       FALSE
set SubnetAddress [regexp -inline {\d{1,3}\.\d{1,3}\.\d{1,3}\.} $DEF_EUT_IP_ADDR ]0
SetIpv4Address  R_Routes_SubnetAddress     $SubnetAddress
SetIpv4Address  R_Routes_SubnetMask        $DEF_EUT_MASK
SetIpv4Address  R_Routes_GatewayAddress    $DEF_EUT_IP_ADDR

Tcp4->Routes {@R_Routes_DeleteRoute, &@R_Routes_SubnetAddress,                 \
              &@R_Routes_SubnetMask, &@R_Routes_GatewayAddress, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_NOT_STARTED]
RecordAssertion $assert $Tcp4RoutesConf1AssertionGuid001                       \
                "Tcp4.Routes() - Call the Routes() when the instance has not   \
                been configured."                                              \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_NOT_STARTED"

#
# Clean up the environment on EUT side.
#
CleanUpEutEnvironment
