/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsUtilityInclude.c
    
Abstract:

    Implementation of Ents TCL commands  "SetCaseRoot" and "Include"

--*/

#include "tcl.h"
#include "EmsUtilityMain.h"

#define MAX_JOIN_PATH 10

STATIC INT8 EmsRootDir[1024] = {0,};

INT32
TclSetCaseRoot (
  IN ClientData      clientData,
  IN Tcl_Interp      *Interp,
  IN INT32           Argc,
  IN CONST84 INT8    *Argv[]
  )
/*++

Routine Description:

  TCL command "SetCaseRoot" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  INT8  ErrorBuff[MAX_ERRBUFF_LEN];
  if (Argc != 2) {
    sprintf (ErrorBuff, "Syntax of SetCaseRoot failed\n");
    goto ErrorExit;
  }

  strcpy(EmsRootDir, Argv[1]);
  return TCL_OK;

ErrorExit:
  Tcl_AppendResult (Interp, ErrorBuff, (INT8 *) NULL);
  return TCL_ERROR;
}

INT32
TclInclude (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "Include" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  INT8        ErrorBuff[MAX_ERRBUFF_LEN];
  Tcl_DString Directory;
  INT32       ArgcJoin;
  INT8        *ArgvJoin[MAX_JOIN_PATH];
  INT32       Index;

  if (Argc > MAX_JOIN_PATH) {
    sprintf (ErrorBuff, "Syntax of Include failed\n");
    goto ErrorExit;
  }

  ArgcJoin    = Argc;
  ArgvJoin[0] = EmsRootDir;
  for (Index = 1; Index < Argc; Index++) {
    ArgvJoin[Index] = (INT8 *) Argv[Index];
  }

  Tcl_DStringInit (&Directory);
  Tcl_JoinPath (ArgcJoin, (CONST84 INT8 **) ArgvJoin, &Directory);
  Tcl_EvalFile (Interp, Tcl_DStringValue (&Directory));
  Tcl_DStringFree (&Directory);

  Tcl_AppendResult (Interp, (INT8 *) NULL);
  return TCL_OK;

ErrorExit:
  Tcl_AppendResult (Interp, ErrorBuff, (INT8 *) NULL);
  return TCL_ERROR;
}
