/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  Debug.c

Abstract:

  Support for Debug primatives.

--*/

#include "lib.h"

EFI_DEBUG_ASSERT_PROTOCOL *mDebugAssert = NULL;

EFI_STATUS
EfiDebugAssertInit (
  VOID
  )
{
  EFI_STATUS  Status;
  EFI_GUID    DebugAssertProtocolGuid = EFI_DEBUG_ASSERT_PROTOCOL_GUID;

  Status = LibLocateProtocol (&DebugAssertProtocolGuid, (VOID **)&mDebugAssert);
  return Status;
}


VOID
EfiDebugAssert (
  IN CHAR8    *FileName,
  IN INTN     LineNumber,
  IN CHAR8    *Description
  )
/*++

Routine Description:

  Worker function for ASSERT(). If Error Logging hub is loaded log ASSERT
  information. If Error Logging hub is not loaded BREAKPOINT().

Arguments:

  FileName    - File name of failing routine.

  LineNumber  - Line number of failing ASSERT().

  Description - Descritption, usally the assertion,

Returns:

  None

--*/
{
  if (mDebugAssert != NULL) {
    mDebugAssert->Assert (mDebugAssert, FileName, LineNumber, Description);
  }
}


VOID
EfiDebugPrint (
  IN  UINTN   ErrorLevel,
  IN  CHAR8   *Format,
  ...
  )
/*++

Routine Description:

  Worker function for DEBUG(). If Error Logging hub is loaded log ASSERT
  information. If Error Logging hub is not loaded do nothing.

Arguments:

  ErrorLevel - If error level is set do the debug print.

  Format     - String to use for the print, followed by Print arguments.

Returns:

  None

--*/
{
  VA_LIST Marker;

  VA_START (Marker, Format);
  if (mDebugAssert != NULL) {
    mDebugAssert->Print (mDebugAssert, ErrorLevel, Format, Marker);
  }
  VA_END (Marker);
}

