///*++
// Copyright (c) 2008, Intel Corporation
// All rights reserved. 
//
// This UEFI SCRT ("Software") is furnished under license and may only be used or copied
// in accordance with the terms of that license UEFI Contributor Agreement.
// No license, express or implied, by estoppel or otherwise, to any intellectual property
// rights is granted by this document. The Software is subject to change without notice,
// and should not be construed as a commitment by Intel Corporation to market, license,
// sell or support any product or technology. Unless otherwise provided for in the license
// under which this Software is provided, the Software is provided AS IS, with no warranties
// of any kind, express or implied. Except as expressly permitted by the Software license,
// neither Intel Corporation nor its suppliers assumes any responsibility or liability
// for any errors or inaccuracies that may appear herein.
//--*/
///*++
//
//Module Name:
//
//  IpfCpuCache.s
//
//--*/ 

.file  "IpfCpuCache.s"

#include  "IpfMacro.i"
#include  "IpfDefines.h"

//-----------------------------------------------------------------------------
//++
// Flush Cache
//
// Arguments : 

// Input = in0 = Starting Address to Flush.
// Input = in1 = Length in bytes.
// Input = b0 = return branch register.
// On Entry :
//
// Return Value: 
//
//  VOID
//  FlushCache (
//    IN UINT64   BaseToFlush,
//    IN UINT64   LengthToFlush
//    );
//
//--
//---------------------------------------------------------------------------
PROCEDURE_ENTRY (FlushCache)

      NESTED_SETUP (5,8,0,0)
            
      mov         loc2 = ar.lc
      
      mov         loc3 = in0                  // Start address.
      mov         loc4 = in1;;                // Length in bytes.
      
      cmp.eq  p6,p7 = loc4, r0;;               // If Length is zero then don't flush any cache
      (p6)  br.spnt.many DoneFlushingC;;         
      
      add         loc4 = loc4,loc3 
      mov         loc5 = 1;;
      sub         loc4 = loc4, loc5 ;; // the End address to flush
                                         
      dep         loc3 = r0,loc3,0,5          
      dep         loc4 = r0,loc4,0,5;;         
      shr         loc3 = loc3,5             
      shr         loc4 = loc4,5;;    // 32 byte cache line
      
      sub         loc4 = loc4,loc3;; // total flush count, It should be add 1 but 
                                     // the br.cloop will first execute one time 
      mov         loc3 = in0                  
      mov         loc5 = 32      
      mov         ar.lc = loc4;;

StillFlushingC:
      fc          loc3;; 
      sync.i;;
      srlz.i;;
      add         loc3 = loc5,loc3;;
      br.cloop.sptk.few StillFlushingC;;

DoneFlushingC:      
      mov         ar.lc = loc2     
      NESTED_RETURN

PROCEDURE_EXIT (FlushCache)

