/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Copyright (c) 2006, Intel Corporation
All rights reserved. 

This UEFI SCT ("Software") is furnished under license and may only be used or copied
in accordance with the terms of that license UEFI Contributor Agreement.
No license, express or implied, by estoppel or otherwise, to any intellectual property
rights is granted by this document. The Software is subject to change without notice,
and should not be construed as a commitment by Intel Corporation to market, license,
sell or support any product or technology. Unless otherwise provided for in the license
under which this Software is provided, the Software is provided AS IS, with no warranties
of any kind, express or implied. Except as expressly permitted by the Software license,
neither Intel Corporation nor its suppliers assumes any responsibility or liability
for any errors or inaccuracies that may appear herein. 

Module Name:
  EventTimerAndTaskPriorityENTSTestCase.c

Abstract:

--*/

#include "EventTimerAndTaskPriorityENTSTestCase.h"

VOID
BSNotifyFunction0 (
  EFI_EVENT Event,
  VOID      *Context
  );

VOID
BSNotifyFunction1 (
  EFI_EVENT Event,
  VOID      *Context
  );

static EFI_EVENT_NOTIFY BSNotifyFunctionList[] = {
  BSNotifyFunction0,
  BSNotifyFunction1
};

//
// gBS.CreateEvent
//
static UINT32           BSCreateEventType;
static EFI_TPL          BSCreateEventNotifyTpl;
static UINTN            BSCreateEventNotifyFunction;
static VOID             *BSCreateEventNotifyContext;
static EFI_EVENT        *BSCreateEventEvent;
static EFI_STATUS       *BSCreateEventStatus;

ENTS_ARG_FIELD          gBSCreateEventArgField[] = {
  {
    OCTET4,
    &BSCreateEventType
  },
  {
    OCTETN,
    &BSCreateEventNotifyTpl
  },
  {
    OCTETN,
    &BSCreateEventNotifyFunction
  },
  {
    POINTER,
    &BSCreateEventNotifyContext
  },
  {
    POINTER,
    &BSCreateEventEvent
  },
  {
    POINTER,
    &BSCreateEventStatus
  },
  0
};

EFI_STATUS
BS_CreateEvent_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  CreateEvent Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSCreateEventStatus = BootService->CreateEvent (
                                        BSCreateEventType,
                                        BSCreateEventNotifyTpl,
                                        BSNotifyFunctionList[BSCreateEventNotifyFunction],
                                        BSCreateEventNotifyContext,
                                        BSCreateEventEvent
                                        );
  return EFI_SUCCESS;
}

//
// gBS.CloseEvent
//
static EFI_EVENT  BSCloseEventEvent;
static EFI_STATUS *BSCloseEventStatus;

ENTS_ARG_FIELD    gBSCloseEventArgField[] = {
  {
    OCTETN,
    &BSCloseEventEvent
  },
  {
    POINTER,
    &BSCloseEventStatus
  },
  0
};

EFI_STATUS
BS_CloseEvent_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  CloseEvent Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService         = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSCloseEventStatus = BootService->CloseEvent (BSCloseEventEvent);
  return EFI_SUCCESS;
}

//
// gBS.SignalEvent
//
static EFI_EVENT  BSSignalEventEvent;
static EFI_STATUS *BSSignalEventStatus;

ENTS_ARG_FIELD    gBSSignalEventArgField[] = {
  {
    OCTETN,
    &BSSignalEventEvent
  },
  {
    POINTER,
    &BSSignalEventStatus
  },
  0
};

EFI_STATUS
BS_SignalEvent_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  SignalEvent Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService           = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSSignalEventStatus  = BootService->SignalEvent (BSSignalEventEvent);
  return EFI_SUCCESS;
}

//
// gBS.WaitForEvent
//
static UINTN      BSWaitForEventNumberOfEvents;
static EFI_EVENT  *BSWaitForEventEvent;
static UINTN      *BSWaitForEventIndex;
static EFI_STATUS *BSWaitForEventStatus;

ENTS_ARG_FIELD    gBSWaitForEventArgField[] = {
  {
    OCTETN,
    &BSWaitForEventNumberOfEvents
  },
  {
    POINTER,
    &BSWaitForEventEvent
  },
  {
    POINTER,
    &BSWaitForEventIndex
  },
  {
    POINTER,
    &BSWaitForEventStatus
  },
  0
};

EFI_STATUS
BS_WaitForEvent_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  WaitForEvent Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSWaitForEventStatus = BootService->WaitForEvent (
                                        BSWaitForEventNumberOfEvents,
                                        BSWaitForEventEvent,
                                        BSWaitForEventIndex
                                        );
  return EFI_SUCCESS;
}

//
// gBS.CheckEvent
//
static EFI_EVENT  BSCheckEventEvent;
static EFI_STATUS *BSCheckEventStatus;

ENTS_ARG_FIELD    gBSCheckEventArgField[] = {
  {
    OCTETN,
    &BSCheckEventEvent
  },
  {
    POINTER,
    &BSCheckEventStatus
  },
  0
};

EFI_STATUS
BS_CheckEvent_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  CheckEvent Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService         = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSCheckEventStatus = BootService->CheckEvent (BSCheckEventEvent);
  return EFI_SUCCESS;
}

//
// gBS.SetTimer
//
static EFI_EVENT        BSSetTimerEvent;
static EFI_TIMER_DELAY  BSSetTimerType;
static UINT64           BSSetTimerTriggerTime;
static EFI_STATUS       *BSSetTimerStatus;

ENTS_ARG_FIELD          gBSSetTimerArgField[] = {
  {
    OCTETN,
    &BSSetTimerEvent
  },
  {
    OCTETN,
    &BSSetTimerType
  },
  {
    OCTET8,
    &BSSetTimerTriggerTime
  },
  {
    POINTER,
    &BSSetTimerStatus
  },
  0
};

EFI_STATUS
BS_SetTimer_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  SetTimer Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSSetTimerStatus = BootService->SetTimer (
                                    BSSetTimerEvent,
                                    BSSetTimerType,
                                    BSSetTimerTriggerTime
                                    );
  return EFI_SUCCESS;
}

VOID
BSNotifyFunction0 (
  EFI_EVENT Event,
  VOID      *Context
  )
/*++

Routine Description:

  Callback function for event notification.

Arguments:

  Event   -  EFI Event.
  Context -  Event context.

Returns:

  None.

--*/
{
  return ;
}

VOID
BSNotifyFunction1 (
  EFI_EVENT Event,
  VOID      *Context
  )
/*++

Routine Description:

  Callback function for event notification.

Arguments:

  Event   -  EFI Event.
  Context -  Event context.

Returns:

  None.

--*/
{
  UINTN *NotifyTimes;

  if (Context == NULL) {
    return ;
  }

  NotifyTimes = (UINTN *) Context;
  (*NotifyTimes)++;

  return ;
}
