/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Copyright (c) 2006, Intel Corporation
All rights reserved. 

This UEFI SCT ("Software") is furnished under license and may only be used or copied
in accordance with the terms of that license UEFI Contributor Agreement.
No license, express or implied, by estoppel or otherwise, to any intellectual property
rights is granted by this document. The Software is subject to change without notice,
and should not be construed as a commitment by Intel Corporation to market, license,
sell or support any product or technology. Unless otherwise provided for in the license
under which this Software is provided, the Software is provided AS IS, with no warranties
of any kind, express or implied. Except as expressly permitted by the Software license,
neither Intel Corporation nor its suppliers assumes any responsibility or liability
for any errors or inaccuracies that may appear herein. 

Module Name:

  TimeENTSTestCase.c

Abstract:

--*/

#include "TimeENTSTestCase.h"

//
// RT.GetTime
//
static EFI_TIME               *RTGetTimeTime;
static EFI_TIME_CAPABILITIES  *RTGetTimeCapabilities;
static EFI_STATUS             *RTGetTimeStatus;

ENTS_ARG_FIELD                gRTGetTimeArgField[] = {
  {
    POINTER,
    &RTGetTimeTime
  },
  {
    POINTER,
    &RTGetTimeCapabilities
  },
  {
    POINTER,
    &RTGetTimeStatus
  },
  0
};

EFI_STATUS
RT_GetTime_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  GetTime Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_RUNTIME_SERVICES  *RuntimeService;

  RuntimeService = (EFI_RUNTIME_SERVICES *) ClientInterface;

  *RTGetTimeStatus = RuntimeService->GetTime (
                                      RTGetTimeTime,
                                      RTGetTimeCapabilities
                                      );
  return EFI_SUCCESS;
}

//
// RT.SetTime
//
static EFI_TIME   *RTSetTimeTime;
static EFI_STATUS *RTSetTimeStatus;

ENTS_ARG_FIELD    gRTSetTimeArgField[] = {
  {
    POINTER,
    &RTSetTimeTime
  },
  {
    POINTER,
    &RTSetTimeStatus
  },
  0
};

EFI_STATUS
RT_SetTime_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  SetTime Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_RUNTIME_SERVICES  *RuntimeService;

  RuntimeService    = (EFI_RUNTIME_SERVICES *) ClientInterface;

  *RTSetTimeStatus  = RuntimeService->SetTime (RTSetTimeTime);
  return EFI_SUCCESS;
}

//
// RT.GetWakeupTime
//
static BOOLEAN    *RTGetWakeupTimeEnabled;
static BOOLEAN    *RTGetWakeupTimePending;
static EFI_TIME   *RTGetWakeupTimeTime;
static EFI_STATUS *RTGetWakeupTimeStatus;

ENTS_ARG_FIELD    gRTGetWakeupTimeArgField[] = {
  {
    POINTER,
    &RTGetWakeupTimeEnabled
  },
  {
    POINTER,
    &RTGetWakeupTimePending
  },
  {
    POINTER,
    &RTGetWakeupTimeTime
  },
  {
    POINTER,
    &RTGetWakeupTimeStatus
  },
  0
};

EFI_STATUS
RT_GetWakeupTime_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  GetWakeupTime Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_RUNTIME_SERVICES  *RuntimeService;

  RuntimeService = (EFI_RUNTIME_SERVICES *) ClientInterface;

  *RTGetWakeupTimeStatus = RuntimeService->GetWakeupTime (
                                            RTGetWakeupTimeEnabled,
                                            RTGetWakeupTimePending,
                                            RTGetWakeupTimeTime
                                            );
  return EFI_SUCCESS;
}

//
// RT.SetWakeupTime
//
static BOOLEAN    RTSetWakeupTimeEnabled;
static EFI_TIME   *RTSetWakeupTimeTime;
static EFI_STATUS *RTSetWakeupTimeStatus;

ENTS_ARG_FIELD    gRTSetWakeupTimeArgField[] = {
  {
    BOOL,
    &RTSetWakeupTimeEnabled
  },
  {
    POINTER,
    &RTSetWakeupTimeTime
  },
  {
    POINTER,
    &RTSetWakeupTimeStatus
  },
  0
};

EFI_STATUS
RT_SetWakeupTime_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  SetWakeupTime Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_RUNTIME_SERVICES  *RuntimeService;

  RuntimeService = (EFI_RUNTIME_SERVICES *) ClientInterface;

  *RTSetWakeupTimeStatus = RuntimeService->SetWakeupTime (
                                            RTSetWakeupTimeEnabled,
                                            RTSetWakeupTimeTime
                                            );
  return EFI_SUCCESS;
}
