/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/**
 *  This file is a test image for the Image Boot Services Test
 */
/*++

Module Name:
    RuntimeServicesDriver4.c

Abstract:       
    for Image Boot Services Black Box Test

--*/

#include "ProtocolDefinition.h"
#include "EfiTestLib.h"

#include EFI_PROTOCOL_DEFINITION (DriverBinding)
#include EFI_PROTOCOL_DEFINITION (DevicePath)
#include EFI_PROTOCOL_DEFINITION (LoadedImage)
//
// data definition here
//
typedef struct {
  EFI_HANDLE                  Handle;
  EFI_HANDLE                  OpenHandle;
  EFI_HANDLE                  ImageHandle;
  EFI_DRIVER_BINDING_PROTOCOL DriverBinding;
} BOOT_SERVICES_DRIVER_4_PRIVATE_DATA;

BOOT_SERVICES_DRIVER_4_PRIVATE_DATA   mPrivateData;

EFI_STATUS
InitializeRuntimeServicesDriver4 (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  );

EFI_STATUS
RuntimeServicesDriver4BindingSupported (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  );

EFI_STATUS
RuntimeServicesDriver4BindingStart (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  );

EFI_STATUS
RuntimeServicesDriver4BindingStop (
  IN  EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN  EFI_HANDLE                     Controller,
  IN  UINTN                          NumberOfChildren,
  IN  EFI_HANDLE                     *ChildHandleBuffer
  );

VOID
InitializeDriverBinding (
  EFI_DRIVER_BINDING_PROTOCOL *DriverBinding
  );
  
EFI_STATUS
RuntimeServicesDriver4Unload (
  IN EFI_HANDLE       ImageHandle
  );

EFI_DRIVER_ENTRY_POINT(InitializeRuntimeServicesDriver4)

EFI_STATUS
InitializeRuntimeServicesDriver4 (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
{
  EFI_STATUS                            Status;  
  EFI_HANDLE                            *HandleBuffer;
  UINTN                                 NoHandles;  
  
  EfiInitializeTestLib (ImageHandle, SystemTable);
  
  //
  // init
  //
  HandleBuffer = NULL;
  gtBS->SetMem (&mPrivateData,sizeof (BOOT_SERVICES_DRIVER_4_PRIVATE_DATA),0);
  
  mPrivateData.ImageHandle = ImageHandle;  
  
  InitializeDriverBinding (&(mPrivateData.DriverBinding));
  
  Status = gtBS->InstallProtocolInterface (
            &ImageHandle,
            &gEfiDriverBindingProtocolGuid,
            EFI_NATIVE_INTERFACE,
            &(mPrivateData.DriverBinding)
            );
  if (EFI_ERROR(Status)) {
    return Status;
  }

  mPrivateData.DriverBinding.ImageHandle = ImageHandle;
  mPrivateData.DriverBinding.DriverBindingHandle = ImageHandle;

  gtBS->InstallProtocolInterface (
                      &mPrivateData.Handle,
                      &mImageTestNoInterfaceProtocol1Guid,
                      EFI_NATIVE_INTERFACE,
                      NULL
                      );
                      
  Status = gtBS->LocateHandleBuffer (
                  ByProtocol,
                  &mImageTestNoInterfaceProtocol2Guid,
                  NULL,
                  &NoHandles,
                  &HandleBuffer
                  );
  if (EFI_ERROR(Status)) {
    return EFI_SUCCESS;
  }
  
  mPrivateData.OpenHandle = HandleBuffer[0];
  gtBS->OpenProtocol (
                mPrivateData.OpenHandle,
                &mImageTestNoInterfaceProtocol2Guid,
                NULL,
                ImageHandle,
                NULL,
                EFI_OPEN_PROTOCOL_TEST_PROTOCOL
                );
                
  gtBS->FreePool (HandleBuffer);              
  
  return EFI_SUCCESS;
}

EFI_STATUS
RuntimeServicesDriver4BindingSupported (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  )  
{
  EFI_STATUS        Status;
  
  
  Status = gtBS->OpenProtocol (
                      Controller,
                      &mImageTestNoInterfaceProtocol3Guid,
                      NULL,
                      This->DriverBindingHandle,
                      Controller,
                      EFI_OPEN_PROTOCOL_TEST_PROTOCOL
                      );
  if (EFI_ERROR(Status)) {
    return EFI_UNSUPPORTED;
  }
  
  return EFI_SUCCESS;  
}

EFI_STATUS
RuntimeServicesDriver4BindingStart (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  )
{
  EFI_STATUS        Status;
  VOID              *ProtInst;

  EFI_LOADED_IMAGE_PROTOCOL             *LoadedImageInfoPtr;
  
  Status = gtBS->OpenProtocol (
                          Controller,
                          &mImageTestNoInterfaceProtocol3Guid,
                          &ProtInst,
                          This->DriverBindingHandle,
                          Controller,
                          EFI_OPEN_PROTOCOL_BY_DRIVER
                          );
  if (EFI_ERROR(Status)) {
    return Status;
  }
  
  //
  // UnLoad Function Handler
  //  
  gtBS->HandleProtocol (
        mPrivateData.ImageHandle, 
        &gEfiLoadedImageProtocolGuid, 
        (VOID*)&LoadedImageInfoPtr
        ); 
        
  LoadedImageInfoPtr->Unload = RuntimeServicesDriver4Unload;
  
  return EFI_SUCCESS;
}

EFI_STATUS
RuntimeServicesDriver4BindingStop (
  IN  EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN  EFI_HANDLE                     Controller,
  IN  UINTN                          NumberOfChildren,
  IN  EFI_HANDLE                     *ChildHandleBuffer
  )
{
  gtBS->CloseProtocol (
              Controller,
              &mImageTestNoInterfaceProtocol3Guid,
              This->DriverBindingHandle,
              Controller
              );
  return EFI_SUCCESS;
}

VOID
InitializeDriverBinding (
  EFI_DRIVER_BINDING_PROTOCOL *DriverBinding
  )
{
  DriverBinding->Supported            = RuntimeServicesDriver4BindingSupported;
  DriverBinding->Start                = RuntimeServicesDriver4BindingStart;
  DriverBinding->Stop                 = RuntimeServicesDriver4BindingStop;
  DriverBinding->Version              = 0x10;
  DriverBinding->ImageHandle          = NULL;
  DriverBinding->DriverBindingHandle  = NULL;
}


EFI_STATUS
RuntimeServicesDriver4Unload (
  IN EFI_HANDLE       ImageHandle
  )
{
  //
  // free resources
  //
  gtBS->UninstallProtocolInterface (
            ImageHandle,
            &gEfiDriverBindingProtocolGuid,
            &(mPrivateData.DriverBinding)
            );
  gtBS->UninstallProtocolInterface (
              mPrivateData.Handle,
              &mImageTestNoInterfaceProtocol1Guid,
              NULL
              );
  gtBS->CloseProtocol (
              mPrivateData.OpenHandle,
              &mImageTestNoInterfaceProtocol2Guid,
              ImageHandle,
              NULL
              );
  
  return EFI_SUCCESS;
}