/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  HIIImageBBTestMain.c

Abstract:

  Test Driver of HII Image Protocol

--*/

#include "HIIImageBBTestMain.h"

//
// Global variables
//
EFI_HANDLE mImageHandle;

EFI_BB_TEST_PROTOCOL *gBBTestProtocolInterface;

EFI_BB_TEST_PROTOCOL_FIELD gBBTestProtocolField = {
  EFI_HII_IMAGE_PROTOCOL_TEST_REVISION,
  EFI_HII_IMAGE_PROTOCOL_GUID,
  L"HII Image Protocol Test",
  L"UEFI 2.1 HII Image Protocol Black-Box Test"
};

EFI_GUID gSupportProtocolGuid[] = {
  EFI_STANDARD_TEST_LIBRARY_GUID,
  EFI_NULL_GUID
};

EFI_BB_TEST_ENTRY_FIELD gBBTestEntryField[] = {
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0101,
    L"NewImageFunction",
    L"Function auto test for HII Image Protocol NewImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestNewImageFunctionTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0102,
    L"GetImageFunction",
    L"Function auto test for HII Image Protocol GetImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestGetImageFunctionTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0103,
    L"SetImageFunction",
    L"Function auto test for HII Image Protocol SetImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestSetImageFunctionTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0104,
    L"DrawImageFunction",
    L"Function auto test for HII Image Protocol DrawImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestDrawImageFunctionTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0105,
    L"DrawImageIdFunction",
    L"Function auto test for HII Image Protocol DrawImageId().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestDrawImageIdFunctionTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0201,
    L"NewImageConformance",
    L"Conformance auto test for HII Image Protocol NewImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestNewImageConformanceTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0202,
    L"GetImageConformance",
    L"Conformance auto test for HII Image Protocol GetImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestGetImageConformanceTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0203,
    L"SetImageConformance",
    L"Conformance auto test for HII Image Protocol SetImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestSetImageConformanceTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0204,
    L"DrawImageConformance",
    L"Conformance auto test for HII Image Protocol DrawImage().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestDrawImageConformanceTest
  },
  {
    EFI_HII_IMAGE_PROTOCOL_TEST_ENTRY_GUID0205,
    L"DrawImageIdConformance",
    L"Conformance auto test for HII Image Protocol DrawImageId().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestDrawImageIdConformanceTest
  },
  0
};

//
//
//
EFI_DRIVER_ENTRY_POINT(InitializeHIIImageBBTest)

EFI_STATUS
InitializeHIIImageBBTest (
  IN EFI_HANDLE                   ImageHandle,
  IN EFI_SYSTEM_TABLE             *SystemTable
  )
{
  EfiInitializeTestLib (ImageHandle, SystemTable);

  //
  // initialize test utility lib
  //

  InitializeLib (ImageHandle, SystemTable);

  mImageHandle = ImageHandle;

  return EfiInitAndInstallBBTestInterface (
           &ImageHandle,
           &gBBTestProtocolField,
           gBBTestEntryField,
           UnloadHIIImageBBTest,
           &gBBTestProtocolInterface
           );
}


EFI_STATUS
UnloadHIIImageBBTest (
  IN EFI_HANDLE                   ImageHandle
  )
{
  return EfiUninstallAndFreeBBTestInterface (
           ImageHandle,
           gBBTestProtocolInterface
           );
}

EFI_STATUS
GetGraphicsOutputInterface (
  OUT EFI_GRAPHICS_OUTPUT_PROTOCOL    **GraphicsOutput
  )
{
  UINTN                 NoHandles;
  EFI_HANDLE            *HandleBuffer;
  EFI_STATUS            Status;
  
  Status = gtBS->LocateHandleBuffer (
                        ByProtocol,
                        &gEfiGraphicsOutputProtocolGuid,
                        NULL,
                        &NoHandles,
                        &HandleBuffer
                        );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }
  
  if ( NoHandles <= 0 ) {
    return EFI_UNSUPPORTED;
  }
  
  Status = gtBS->HandleProtocol (
             HandleBuffer[0],
             &gEfiGraphicsOutputProtocolGuid,
             GraphicsOutput
             );
  if ( EFI_ERROR(Status) ) {
    gtBS->FreePool( HandleBuffer );
    return Status;
  }
  
  gtBS->FreePool( HandleBuffer );
  
  return EFI_SUCCESS;
}

EFI_STATUS
GetHIIDatabaseInterface (
  OUT EFI_HII_DATABASE_PROTOCOL    **HIIDatabase
  )
{
  UINTN                                 NoHandles;
  EFI_HANDLE                            *HandleBuffer;
  EFI_STATUS                            Status;
  
  //
  // Get the HII Database Protocol interface
  //
  Status = gtBS->LocateHandleBuffer (
                        ByProtocol,
                        &gEfiHIIDatabaseProtocolGuid,
                        NULL,
                        &NoHandles,
                        &HandleBuffer
                        );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }
  
  if ( NoHandles <= 0 ) {
    return EFI_DEVICE_ERROR;
  }
  
  Status = gtBS->HandleProtocol (
             HandleBuffer[0],
             &gEfiHIIDatabaseProtocolGuid,
             HIIDatabase
             );
  if ( EFI_ERROR(Status) ) {
    gtBS->FreePool( HandleBuffer );
    return Status;
  }
  
  gtBS->FreePool( HandleBuffer );
  
  return EFI_SUCCESS;
}