/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  HIIStringBBTestMain.h

Abstract:

  Test Driver of HII String Protocol header file

--*/

#ifndef _HII_STRING_BB_TEST_MAIN
#define _HII_STRING_BB_TEST_MAIN


#include "Efi.h"
#include "EfiTest.h"
#include "EfiTestLib.h"
//#include "EfiTestUtilityLib.h"
#include "EfiCommonLib.h"
#include "HiiDef.h"
#include "HiiProtocol.h"
#include "HIISTRING.h"
#include "Guid.h"


#define EFI_HII_STRING_PROTOCOL_TEST_REVISION    0x00010000

//
// Entry GUIDs for Func Test
//
#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0101 \
{ 0xf5050750, 0x8e84, 0x46db, { 0xa3, 0xe3, 0x62, 0x7, 0x34, 0x13, 0xa5, 0x69 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0102 \
{ 0x4dfdbdfc, 0xb23f, 0x49db, { 0x91, 0x70, 0x7d, 0x69, 0xfd, 0xf8, 0x88, 0x68 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0103 \
{ 0x2ec82b5d, 0x49f7, 0x4b3e, { 0xb8, 0x84, 0xa5, 0xeb, 0x17, 0x1e, 0x23, 0x69 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0104 \
{ 0x5ce20b3c, 0xcb59, 0x412f, { 0xa1, 0x37, 0xbb, 0xe5, 0x94, 0x22, 0xf8, 0x60 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0105 \
{ 0x75a47073, 0x8ba9, 0x4b56, { 0x8d, 0xf1, 0x54, 0xa1, 0x00, 0x63, 0xf6, 0xa1 } }

//
// Entry GUIDs for Conf Test
//
#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0201 \
{ 0x94971a6a, 0xb0eb, 0x4c86, { 0xb3, 0x4a, 0x86, 0x68, 0x86, 0x80, 0x1d, 0xf9 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0202 \
{ 0x9d28a801, 0x49d8, 0x4a7f, { 0x89, 0x5e, 0x73, 0x2e, 0x43, 0x6b, 0xdf, 0x3c } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0203 \
{ 0x69ddddbb, 0xdce4, 0x4afc, { 0x90, 0x60, 0x26, 0xb4, 0x99, 0x8e, 0x7d, 0xc1 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0204 \
{ 0x24e0cca8, 0x80f0, 0x4391, { 0x88, 0x16, 0x3b, 0xcc, 0x21, 0x61, 0x6e, 0x54 } }

#define EFI_HII_STRING_PROTOCOL_TEST_ENTRY_GUID0205 \
{ 0xaa37eddc, 0x68ee, 0x4d69, { 0x8d, 0x0b, 0xa4, 0x31, 0xf1, 0xe7, 0xb8, 0x0f } }

//
//
//
EFI_STATUS
InitializeHIIStringBBTest (
  IN EFI_HANDLE                   ImageHandle,
  IN EFI_SYSTEM_TABLE             *SystemTable
  );
  
  
EFI_STATUS
UnloadHIIStringBBTest (
  IN EFI_HANDLE                   ImageHandle
  );
  
  
//
//Test Cases
//
EFI_STATUS
BBTestNewStringConformanceTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestGetStringConformanceTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestSetStringConformanceTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestGetLanguagesConformanceTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );

EFI_STATUS
BBTestGetSecondaryLanguagesConformanceTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestNewStringFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestGetStringFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestSetStringFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );
  
EFI_STATUS
BBTestGetLanguagesFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );

EFI_STATUS
BBTestGetSecondaryLanguagesFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  );

//
//
//
EFI_STATUS
GetHIIDatabaseInterface (
  OUT EFI_HII_DATABASE_PROTOCOL    **HIIDatabase
  );
  
VOID
InitializeLib (
  IN EFI_HANDLE                     ImageHandle,
  IN EFI_SYSTEM_TABLE               *SystemTable
  );
  
VOID *
AllocatePool (
  IN UINTN                          Size
  );

VOID *
AllocateZeroPool (
  IN UINTN                          Size
  );
  
INTN
CompareMem (
  IN VOID                           *Dest,
  IN VOID                           *Src,
  IN UINTN                          len
  );

#endif
