/*++
 Copyright (c) 2008, Byosoft.NJ
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  PlatformToDriverConfigurationProtocol.h

Abstract:

  Platform To Driver Configuration protocol as defined in the UEFI 2.10 specification.

--*/


#ifndef _PLATFORM_TO_DRIVER_CONFIGURATION_FOR_TEST_H_
#define _PLATFORM_TO_DRIVER_CONFIGURATION_FOR_TEST_H_

//
// Global ID for the Platform To Driver Configuration Protocol
//
#define EFI_PLATFORM_TO_DRIVER_CONFIGURATION_PROTOCOL_GUID \
    { 0x642cd590, 0x8059, 0x4c0a, { 0xa9, 0x58, 0xc5, 0xec, 0x7, 0xd2, 0x3c, 0x4b} }


EFI_FORWARD_DECLARATION(EFI_PLATFORM_TO_DRIVER_CONFIGURATION_PROTOCOL);

#define EFI_PLATFORM_TO_DRIVER_CONFIGURATION_CLP_GUID \
  { \
    0x345ecc0e, 0x0cb6, 0x4b75, 0xbb, 0x57, 0x1b, 0x12, 0x9c, 0x47, 0x33, 0x3e \
  }
 //
 //Related Definitions
 //
typedef enum { 
  EfiPlatformConfigurationActionNone                                     = 0, 
  EfiPlatformConfigurationActionStopController                           = 1, 
  EfiPlatformConfigurationActionRestartController                        = 2, 
  EfiPlatformConfigurationActionRestartPlatform                          = 3, 
  EfiPlatformConfigurationActionNvramFailed                              = 4, 
  EfiPlatformConfigurationActionMaximum 
} EFI_PLATFORM_CONFIGURATION_ACTION;


//
// Parameter Block Definition
//
typedef struct {
  CHAR8          *CLPCommand;
  UINT32         CLPCommandLength;
  CHAR8          *CLPReturnString;
  UINT32         CLPReturnStringLength;
  UINT8           CLPCmdStatus;
  UINT8           CLPErrorValue;
  UINT16         CLPMsgCode;
} EFI_CONFIGURE_CLP_PARAMETER_BLK;

//extern EFI_GUID gEfiParameterBlockGuid;

//
// Prototypes for the Platform To Driver Configuration Protocol
//

typedef
EFI_STATUS
(EFIAPI *EFI_PLATFORM_TO_DRIVER_CONFIGURATION_QUERY) (
  IN  EFI_PLATFORM_TO_DRIVER_CONFIGURATION_PROTOCOL          *This,
  IN  EFI_HANDLE                                             ControllerHandle,
  IN  EFI_HANDLE                                             ChildHandle OPTIONAL,
  IN  UINTN                                               *Instance,
  OUT EFI_GUID                                            **ParameterTypeGuid,
  OUT VOID                                                **ParameterBlock,
  OUT UINTN                                               *ParameterBlockSize
  );

typedef
EFI_STATUS
(EFIAPI *EFI_PLATFORM_TO_DRIVER_CONFIGURATION_RESPONSE) (
  IN  EFI_PLATFORM_TO_DRIVER_CONFIGURATION_PROTOCOL          *This,
  IN  EFI_HANDLE                                             ControllerHandle,
  IN  EFI_HANDLE                                             ChildHandle OPTIONAL,
  IN  UINTN                                                  *Instance,
  IN  EFI_GUID                                               *ParameterTypeGuid,
  IN  VOID                                                   *ParameterBlock,
  IN  UINTN                                                  ParameterBlockSize,
  IN  EFI_PLATFORM_CONFIGURATION_ACTION                      ConfigurationAction
  );

//
// Interface structure for the Platform To Driver Configuration Protocol
//
typedef struct _EFI_PLATFORM_TO_DRIVER_CONFIGURATION_PROTOCOL {
  EFI_PLATFORM_TO_DRIVER_CONFIGURATION_QUERY       Query;
  EFI_PLATFORM_TO_DRIVER_CONFIGURATION_RESPONSE    Response;
} EFI_PLATFORM_TO_DRIVER_CONFIGURATION_PROTOCOL;

extern EFI_GUID gEfiPlatformToDriverConfigurationProtocolGuid;
extern EFI_GUID gEfiPlatformToDriverConfigurationClpGuid;


#endif /* _PLATFORM_TO_DRIVER_CONFIGURATION_FOR_TEST_H_ */
