/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  TestProfile.h

Abstract:

  Driver to publish the Test Profile Library Protocol.

--*/

#ifndef _EFI_TEST_PROFILE_H_
#define _EFI_TEST_PROFILE_H_

//
// Includes
//

#include "Efi.h"
#include "EfiTest.h"
#include "LibPrivate.h"
#include EFI_PROTOCOL_DEFINITION (LoadedImage)
#include EFI_PROTOCOL_DEFINITION (SimpleFileSystem)
#include EFI_PROTOCOL_DEFINITION (DevicePath)
#include EFI_TEST_PROTOCOL_DEFINITION (TSLInit)
#include EFI_TEST_PROTOCOL_DEFINITION (TestProfileLibrary)

//
// Private Data definition for TestProfileLibrary protocol
//
#define TEST_PROFILE_PRIVATE_DATA_SIGNATURE   EFI_SIGNATURE_32('A','D','V','L')

typedef struct {
  UINT32                                    Signature;
  EFI_TEST_PROFILE_LIBRARY_PROTOCOL         TestProfile;
  EFI_DEVICE_PATH_PROTOCOL                  *DevicePath;
  CHAR16                                    *FilePath;
} TEST_PROFILE_PRIVATE_DATA;

#define TEST_PROFILE_PRIVATE_DATA_FROM_THIS(a)  \
  CR(a, TEST_PROFILE_PRIVATE_DATA, TestProfile, TEST_PROFILE_PRIVATE_DATA_SIGNATURE)

//
// Private Data definition for EFI_INI_FILE interface
//
#define EFI_INI_FILE_PRIVATE_DATA_SIGNATURE   EFI_SIGNATURE_32('I','N','I','F')

#define MAX_STRING_LEN      250
#define MAX_LINE_LEN        512

//
// Forward reference for pure ANSI compatibility
//

EFI_FORWARD_DECLARATION (INI);
EFI_FORWARD_DECLARATION (COMMENTLINE);

typedef struct _INI {
  UINT32                          commentNo;
  CHAR8                           *ptrSection;
  CHAR8                           *ptrEntry;
  CHAR8                           *ptrValue;
  INI                             *ptrNext;
} INI;


typedef struct _COMMENTLINE {
  UINT32                          commentNo;
  CHAR8                           *ptrComment;
  COMMENTLINE                     *ptrNext;
} COMMENTLINE;

typedef struct {
  UINT32                          Signature;
  EFI_INI_FILE                    Handle;
  EFI_DEVICE_PATH_PROTOCOL        *DevPath;
  CHAR16                          *FileName;
  INI                             *Head;
  INI                             *Tail;
  COMMENTLINE                     *CommentLineHead;
  BOOLEAN                         isUnicode;
  BOOLEAN                         Modified;
} EFI_INI_FILE_PRIVATE_DATA;

#define EFI_INI_FILE_PRIVATE_DATA_FROM_THIS(a)  \
  CR(a, EFI_INI_FILE_PRIVATE_DATA, Handle, EFI_INI_FILE_PRIVATE_DATA_SIGNATURE)

#endif