/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  EntsResult.c
  
Abstract:
  
  Implementation of recording result services 
  
--*/

#include "EntsLib.h"

EFI_STATUS
RecordMessage (
  IN  OUT CHAR16**ResultBuffer,
  IN  OUT UINTN *ResultBufferSize,
  IN     CHAR16 *Format,
  ...
  )
/*++

Routine Description:

  Record runtime information to a buffer.

Arguments:

  ResultBuffer      - Buffer space.
  ResultBufferSize  - Result buffer size in octets.
  Format            - Format string.
  ...               - Variables.

Returns:

  EFI_SUCCESS - Operation succeeded.
  EFI_OUT_OF_RESOURCES - Memory allocation failed.

--*/
{
  UINTN   Size;
  VA_LIST Marker;
  CHAR16  *Buffer;

  Buffer = EntsAllocatePool (ENTS_MAX_BUFFER_SIZE * 4 * 8);
  if (Buffer == NULL) {
    EFI_ENTS_DEBUG ((EFI_ENTS_D_ERROR, L"Error in RecordMessage Status - EFI_OUT_OF_RESOURCES"));
    return EFI_OUT_OF_RESOURCES;
  }

  VA_START (Marker, Format);
  EntsVSPrint (Buffer, ENTS_MAX_BUFFER_SIZE * 4 * 8, Format, Marker);
  VA_END (Marker);

  if (*ResultBuffer == NULL) {
    Size          = EntsStrLen (Buffer) * 2 + 4 + ENTS_MAX_BUFFER_SIZE * 4;
    *ResultBuffer = EntsAllocatePool (Size);
    if (*ResultBuffer == NULL) {
      EFI_ENTS_DEBUG ((EFI_ENTS_D_ERROR, L"Error in RecordMessage Status - EFI_OUT_OF_RESOURCES"));
      goto RecordMessageFail1;
    }
    (*ResultBuffer)[0]  = L'\0';
    *ResultBufferSize   = Size;
  }

  Size = EntsStrLen (*ResultBuffer) * 2 + EntsStrLen (Buffer) * 2 + 4;
  if ((*ResultBufferSize) < Size) {
    *ResultBuffer = EntsReallocatePool (*ResultBuffer, *ResultBufferSize, Size + ENTS_MAX_BUFFER_SIZE * 4);
    if (*ResultBuffer == NULL) {
      EFI_ENTS_DEBUG ((EFI_ENTS_D_ERROR, L"Error in RecordMessage Status - EFI_OUT_OF_RESOURCES"));
      goto RecordMessageFail2;
    }

    *ResultBufferSize = Size + ENTS_MAX_BUFFER_SIZE * 4;
  }

  EntsStrCat (*ResultBuffer, Buffer);
  EntsStrCat (*ResultBuffer, L"\n");

  EntsFreePool (Buffer);
  return EFI_SUCCESS;

RecordMessageFail2:
  if(*ResultBuffer != NULL) {
    EntsFreePool (*ResultBuffer);
  }
RecordMessageFail1:
  if (Buffer != NULL) {
    EntsFreePool (Buffer);
  }

  return EFI_OUT_OF_RESOURCES;
}
