/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  RivlVariable.h

Abstract:

  Remote Interface Validation Language Variable definitions

--*/

#ifndef __RIVL_VARIABLE_H__
#define __RIVL_VARIABLE_H__

#include "Rivl.h"

EFI_STATUS
AddRivlVariable (
  IN CHAR16                    *Name,
  IN CHAR16                    *Type,
  IN UINT32                    TotalSize,
  IN UINT32                    ArrayNumber,
  IN UINT32                    Attribute
  )
/*++

Routine Description:

  This func is to add RivlVariable to gRivlVariableList.

Arguments:

  Name        - Type of RivlVariable.
  Type        - Type of RivlVariable.
  TotalSize   - TotalSize of RivlVariable.
  ArrayNumber - ArrayNumber of RivlVariable.
  Attribute   - Attribute of RivlVariable.

Returns:

  EFI_ALREADY_STARTED   - Rivil type already existed.
  EFI_OUT_OF_RESOURCES  - Memory allocation failed.
  EFI_INVALID_PARAMETER - Invalide parameter.
  EFI_SUCCESS           - Operation succeeded.
  
--*/
;

EFI_STATUS
DelRivlVariable (
  IN CHAR16                    *Name
  )
/*++

Routine Description:

  This func is to delete RivlVariable from gRivlVariableList.

Arguments:

  Name  - Type of RivlVariable.

Returns:

  EFI_INVALID_PARAMETER - Parameter invalid.
  EFI_NOT_FOUND         - The specified type cannot be found.
  EFI_SUCCESS           - Operation succeeded.

--*/
;

EFI_STATUS
DelRivlAllVariable (
  VOID
  )
/*++

Routine Description:

  This func is to delete all RivlVariable from gRivlVariableList.

Arguments:

  None

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
;

#endif // __RIVL_VARIABLE_H__
